#include "service.h"

extern void serviceStop(void );
extern void serviceStart(int argc, char **argv);

#ifdef WIN32
SERVICE_STATUS serviceStatus;
SERVICE_STATUS_HANDLE serviceStatusHandle;
DWORD dwErr=0;
#endif

void WINAPI serviceMain(DWORD argc, LPTSTR *argv)
{
	serviceStatusHandle=RegisterServiceCtrlHandler( TEXT(SERVICENAME), serviceCtrl);

	if (!serviceStatusHandle)
		return;

	serviceStatus.dwServiceType=SERVICE_WIN32_OWN_PROCESS;
	serviceStatus.dwServiceSpecificExitCode=0;

	if (ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000))
		serviceStart(argc, argv);

	ReportStatusToSCMgr(SERVICE_STOPPED, dwErr, 0 );

}

bool ReportStatusToSCMgr(DWORD dwCurrentState, DWORD dwWin32ExitCode, DWORD dwWaitHint)
{
	static DWORD dwCheckPoint = 1;
	bool fResult = TRUE;

	if (dwCurrentState == SERVICE_START_PENDING)
		serviceStatus.dwControlsAccepted = 0;
	else
		serviceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP;

	serviceStatus.dwCurrentState = dwCurrentState;
	serviceStatus.dwWin32ExitCode = dwWin32ExitCode;
	serviceStatus.dwWaitHint = dwWaitHint;

	if ( ( dwCurrentState == SERVICE_RUNNING ) || ( dwCurrentState == SERVICE_STOPPED ) )
		serviceStatus.dwCheckPoint = 0;
	else
		serviceStatus.dwCheckPoint = dwCheckPoint++;


	// Report the status of the service to the service control manager.
	//
	if (!(fResult = SetServiceStatus( serviceStatusHandle, &serviceStatus)))
		AddToMessageLog(TEXT("SetServiceStatus"),EVENTLOG_INFORMATION_TYPE);

	return fResult;
}

//Install service in system
bool installService()
{
SC_HANDLE manager;
SC_HANDLE service = NULL;

char fileName[512];

	//Get service filename
	GetModuleFileName( NULL, fileName, 512 );


	manager=OpenSCManager(
		NULL,	// pointer to machine name string NULL=default
		NULL,  // pointer to database name string NULL=default
		SC_MANAGER_ALL_ACCESS	 // type of access
	);

	if (manager)
	{
		service=CreateService(
			manager,			// handle to SCM database
			TEXT(SERVICENAME),			// name of service to start
			TEXT(SERVICEDISPLAYNAME),	// display name
			SERVICE_ALL_ACCESS,	// type of access to service
			SERVICE_WIN32_OWN_PROCESS,	// type of service
			SERVICE_AUTO_START,	// when to start service
			SERVICE_ERROR_NORMAL,	// severity of service failure
			fileName,		// name of binary file
			NULL,			// name of load ordering group
			NULL,			// receives tag identifier
			NULL,			// array of dependency names
			NULL,			// service account name
			NULL				// account password
		);
	}

	if (service)
		return true;
	else return false;
}

//Remove service from system

bool removeService()
{
SC_HANDLE manager;
SC_HANDLE service;

	manager=OpenSCManager(
		NULL,	// pointer to machine name string NULL=default
		NULL,  // pointer to database name string NULL=default
		SC_MANAGER_ALL_ACCESS	 // type of access
	);

	if (manager)
	{
		service=OpenService(
			manager,	// handle to service control manager database
			TEXT(SERVICENAME), // pointer to name of service to start
			SERVICE_ALL_ACCESS	// type of access to service
		);

		if (service)
		{

			if (ControlService(service, SERVICE_CONTROL_STOP, &serviceStatus ))
			{
				Sleep(1000);

				while ( QueryServiceStatus( service, &serviceStatus) )
				{
					if (serviceStatus.dwCurrentState==SERVICE_STOP_PENDING)
					{
						printf(".");
						Sleep(1000);
					}
					else
						break;
				}
				if (serviceStatus.dwCurrentState==SERVICE_STOPPED)

					MessageBox(NULL, "\n%s stopped.\n",TEXT(SERVICEDISPLAYNAME),MB_OK);
				else
					MessageBox(NULL, "\n%s failed to stop.\n",TEXT(SERVICEDISPLAYNAME), MB_OK);
			}

			if (DeleteService(service))
				MessageBox(NULL, "PostgreSQL removed.\n",TEXT(SERVICEDISPLAYNAME), MB_OK);
			else
				MessageBox(NULL , "DeleteService failed\n", TEXT(SERVICEDISPLAYNAME), MB_OK);

			CloseServiceHandle(service);

		}
		CloseServiceHandle(manager);
	}
	return true;
}



void WINAPI serviceCtrl(DWORD dwCtrlCode)
{

	switch (dwCtrlCode)
	{
		case SERVICE_CONTROL_STOP:
			ReportStatusToSCMgr( SERVICE_STOP_PENDING, NO_ERROR, 10000 );
			serviceStop();
			ReportStatusToSCMgr( SERVICE_STOPPED, NO_ERROR, 10000 );
			return;
		case SERVICE_CONTROL_INTERROGATE:
			break;
		default:
			break;
	}
	ReportStatusToSCMgr(serviceStatus.dwCurrentState, NO_ERROR, 0);
}

//Function adds entry to Windows event log
// Params:
// LPTSTR message: this message will be added to log
// DWORD informationtype : info kind possibly values are
// EVENTLOG_ERROR_TYPE 1
// EVENTLOG_WARNING_TYPE 2
// EVENTLOG_INFORMATION_TYPE 4
// see winbase.h
void AddToMessageLog(LPTSTR message,DWORD informationtype)
{
TCHAR MSG[256];
HANDLE eventSource;
const char *strings[2];

	dwErr=GetLastError();
	eventSource=RegisterEventSource( NULL, TEXT(SERVICENAME) );

	_stprintf(MSG, TEXT("%s error: %ld"), TEXT(SERVICENAME), dwErr);
	strings[0]=MSG;
	strings[1]=message;

	if (eventSource!=NULL)
	{
		ReportEvent(eventSource,
			informationtype,
			0,
			0,
			NULL,
			2,
			0,
			strings,
			NULL);

		(void) DeregisterEventSource(eventSource);
	}

}
